# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/deployment_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.features import deployment_pb2 as qwak_dot_feature__store_dot_features_dot_deployment__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4qwak/feature_store/features/deployment_service.proto\x12\x1bqwak.feature.store.features\x1a,qwak/feature_store/features/deployment.proto\"\x87\x01\n%PushEnvironmentDeploymentStateRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\x12\x46\n\x10\x64\x65ployment_state\x18\x02 \x03(\x0b\x32,.qwak.feature.store.features.DeploymentState\"(\n&PushEnvironmentDeploymentStateResponse2\xc9\x01\n\x1b\x44\x65ploymentManagementService\x12\xa9\x01\n\x1ePushEnvironmentDeploymentState\x12\x42.qwak.feature.store.features.PushEnvironmentDeploymentStateRequest\x1a\x43.qwak.feature.store.features.PushEnvironmentDeploymentStateResponseB]\n(com.qwak.ai.feature.store.management.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')



_PUSHENVIRONMENTDEPLOYMENTSTATEREQUEST = DESCRIPTOR.message_types_by_name['PushEnvironmentDeploymentStateRequest']
_PUSHENVIRONMENTDEPLOYMENTSTATERESPONSE = DESCRIPTOR.message_types_by_name['PushEnvironmentDeploymentStateResponse']
PushEnvironmentDeploymentStateRequest = _reflection.GeneratedProtocolMessageType('PushEnvironmentDeploymentStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _PUSHENVIRONMENTDEPLOYMENTSTATEREQUEST,
  '__module__' : 'qwak.feature_store.features.deployment_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.PushEnvironmentDeploymentStateRequest)
  })
_sym_db.RegisterMessage(PushEnvironmentDeploymentStateRequest)

PushEnvironmentDeploymentStateResponse = _reflection.GeneratedProtocolMessageType('PushEnvironmentDeploymentStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _PUSHENVIRONMENTDEPLOYMENTSTATERESPONSE,
  '__module__' : 'qwak.feature_store.features.deployment_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.PushEnvironmentDeploymentStateResponse)
  })
_sym_db.RegisterMessage(PushEnvironmentDeploymentStateResponse)

_DEPLOYMENTMANAGEMENTSERVICE = DESCRIPTOR.services_by_name['DeploymentManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n(com.qwak.ai.feature.store.management.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _PUSHENVIRONMENTDEPLOYMENTSTATEREQUEST._serialized_start=132
  _PUSHENVIRONMENTDEPLOYMENTSTATEREQUEST._serialized_end=267
  _PUSHENVIRONMENTDEPLOYMENTSTATERESPONSE._serialized_start=269
  _PUSHENVIRONMENTDEPLOYMENTSTATERESPONSE._serialized_end=309
  _DEPLOYMENTMANAGEMENTSERVICE._serialized_start=312
  _DEPLOYMENTMANAGEMENTSERVICE._serialized_end=513
# @@protoc_insertion_point(module_scope)



from typing import Dict, List

from qwak.clients.feature_store import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.feature_store._common.feature_set_utils import (
    FeatureSetInfo,
    get_entity_type,
    get_feature_set_info,
    get_typed_feature,
)
from qwak.model.schema_entities import BaseFeature, Entity, FeatureStoreInput


def get_feature_set_name(feature: BaseFeature) -> str:
    feature_full_name: List[str] = feature.name.split(".")
    return feature_full_name[0].lower()


def normalize_features(
    features: List[BaseFeature], feature_set_to_entities: Dict[str, FeatureSetInfo]
) -> List[BaseFeature]:
    """
    Adding the relevant entity to the feature set features
    Args:
        features: list of features
        feature_set_to_entities: dict of feature set name and it's entity

    return normalize features - the features with the entities
    """
    new_normalize_features = []
    for feature in features:
        if isinstance(feature, FeatureStoreInput):
            feature_set_name = get_feature_set_name(feature)
            entity_spec = feature_set_to_entities.get(feature_set_name).entity_spec
            retrieved_entity = Entity(
                name=entity_spec.keys[
                    0
                ],  # currently support only one entity key per feature set
                type=get_entity_type(entity_spec.value_type),
            )
            if (
                feature.entity
                and retrieved_entity.name.lower() != feature.entity.name.lower()
            ):
                raise QwakException(
                    f"Explicitly supplied with an invalid entity: {feature.entity}, "
                    f"actual: {retrieved_entity}"
                )
            feature.entity = retrieved_entity
            feature.entity.name = feature.entity.name.lower()
            if isinstance(feature, FeatureStoreInput):
                feature_type = feature_set_to_entities.get(
                    feature_set_name
                ).feature_set_type
                feature_version = feature_set_to_entities.get(
                    feature_set_name
                ).feature_version
                feature = get_typed_feature(feature, feature_type, feature_version)
        new_normalize_features.append(feature)

    return new_normalize_features


def adding_entities_to_schema(entities: List[Entity], features: List[BaseFeature]):
    """
    Adding the new entities for feature store feature to the entities list from schema
    Args:
        entities: list of entities
        features: list of features
    Returns: the entities with the new entities from the feature store features
    """
    entity_name_to_entity = {entity.name: entity for entity in entities}
    for feature in features:
        if isinstance(feature, FeatureStoreInput):
            entity_name_to_entity[feature.entity.name] = feature.entity

    for entity in entity_name_to_entity.values():
        entity.name = entity.name.lower()

    return list(set(entity_name_to_entity.values()))


def normalize_schema(features: List[BaseFeature], entities: List[Entity]):
    """
    Normalize schema - will add entity to each features in schema if not exists
    Arg:
        model: the model with the required schema
    Return:
        updated model
    """

    normalized_features = features
    normalized_entities = entities
    if features and hasattr(features, "__iter__"):
        # featureset_name -> FeatureSetInfo
        fs_info_cache: Dict[str, FeatureSetInfo] = dict()
        features_manager_client = FeatureRegistryClient()
        validation_errors = []
        for feature in features:
            try:
                if isinstance(feature, FeatureStoreInput):
                    feature_set_name = get_feature_set_name(feature)
                    if feature_set_name not in fs_info_cache:
                        fs_info_cache[feature_set_name] = get_feature_set_info(
                            features_manager_client, feature_set_name
                        )
                else:
                    fs_info_cache[feature.name] = feature
            except QwakException as e:
                validation_errors.append(e.message)

        if validation_errors:
            raise QwakException(f"Schema validation error: {validation_errors}")
        normalized_features = normalize_features(features, fs_info_cache)
        normalized_entities = adding_entities_to_schema(entities, features)

    return normalized_features, normalized_entities


def enrich_schema(features: List[BaseFeature], entities: List[Entity]):
    normalized_features, normalized_entities = normalize_schema(features, entities)
    return normalized_features, normalized_entities



from typing import Any, Sequence

from .base_input_adapter import BaseInputAdapter


class TfTensorInputAdapter(BaseInputAdapter):
    def extract_user_func_arg(self, data: str) -> Sequence[Any]:
        pass



from typing import Any, List, Sequence

from .base_input_adapter import BaseInputAdapter
from .proto_input_adapter import ProtoInputAdapter

try:
    from bentoml.adapters import JsonInput
    from bentoml.types import InferenceTask
except ImportError:
    pass
    # bentoml is supported either by supplying it as a dependency (or sub-dependency)
    # in your Python project, or during the runtime


class MultiInputAdapter(BaseInputAdapter):
    def __init__(self, adapters: List[BaseInputAdapter]):
        self.adapters = adapters

    def extract_user_func_arg(self, data: bytes) -> Sequence[Any]:
        task = InferenceTask(data=data)
        for inner_adapter in self.adapters:
            try:
                if isinstance(inner_adapter, ProtoInputAdapter):
                    adapter_impl = inner_adapter
                    extracted_data = inner_adapter.extract_user_func_arg(data)
                else:
                    if isinstance(inner_adapter, type):
                        adapter_impl = self.mappings.get(inner_adapter)
                    else:
                        adapter_impl = self.mappings.get(inner_adapter.__class__)

                    extracted_data = adapter_impl().extract_user_func_args([task])

                if isinstance(extracted_data, tuple):
                    extracted_data = extracted_data[0]
                if extracted_data is None:
                    raise ValueError("Parser returned None")

                if adapter_impl == JsonInput and extracted_data == [] and len(data) > 3:
                    # JsonParser return an empty array when it cannot parse the input.
                    # This if raises an exception when the returned [] is caused by an error
                    # (the input isn't an empty array)
                    raise ValueError(
                        "JsonParser returned an empty array when the data is too long for an empty array"
                    )
                return extracted_data
            except Exception as e:
                print(f"{inner_adapter} skipped an adapter because of: {e}")

        raise ValueError(f"Cannot parse with given adapters: {self.adapters}")



#
#    Copyright 2024 OpenAI
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ...types import shared_params

__all__ = ["ChatCompletionToolParam"]


class ChatCompletionToolParam(TypedDict, total=False):
    function: Required[shared_params.FunctionDefinition]

    type: Required[Literal["function"]]
    """The type of the tool. Currently, only `function` is supported."""



from enum import Enum, auto

from _qwak_proto.qwak.feature_store.features.execution_pb2 import (
    LARGE as ProtoLARGE,
    MEDIUM as ProtoMEDIUM,
    NANO as ProtoNANO,
    SMALL as ProtoSMALL,
    XLARGE as ProtoXLARGE,
    XXLARGE as ProtoXXLARGE,
    XXXLARGE as ProtoXXXLARGE,
    ClusterTemplate as ProtoClusterTemplate,
)


class AutoName(Enum):
    def _generate_next_value_(name, start, count, last_values):
        return name


class ClusterTemplate(AutoName):
    SMALL = auto()
    MEDIUM = auto()
    LARGE = auto()
    XLARGE = auto()
    XXLARGE = auto()
    XXXLARGE = auto()
    NANO = auto()

    _cluster_template_to_proto = {
        SMALL: ProtoSMALL,
        MEDIUM: ProtoMEDIUM,
        LARGE: ProtoLARGE,
        XLARGE: ProtoXLARGE,
        XXLARGE: ProtoXXLARGE,
        XXXLARGE: ProtoXXXLARGE,
        NANO: ProtoNANO,
    }

    _proto_to_cluster_template = {v: k for k, v in _cluster_template_to_proto.items()}

    @classmethod
    def from_proto(cls, proto: ProtoClusterTemplate):
        return cls(
            ClusterTemplate._proto_to_cluster_template.value[proto.cluster_template]
        )

    @staticmethod
    def to_proto(template):
        if not template:
            return None

        _cluster_template_to_proto = {
            ClusterTemplate.SMALL: ProtoSMALL,
            ClusterTemplate.MEDIUM: ProtoMEDIUM,
            ClusterTemplate.LARGE: ProtoLARGE,
            ClusterTemplate.XLARGE: ProtoXLARGE,
            ClusterTemplate.XXLARGE: ProtoXXLARGE,
            ClusterTemplate.XXXLARGE: ProtoXXXLARGE,
            ClusterTemplate.NANO: ProtoNANO,
        }
        return _cluster_template_to_proto[template]



from qwak.feature_store.feature_sets.transformations.aggregations.aggregations import (
    QwakAggregation,
)
from qwak.feature_store.feature_sets.transformations.aggregations.windows import Window
from qwak.feature_store.feature_sets.transformations.functions import (
    Column,
    Schema,
    Type,
    qwak_pandas_udf,
)
from qwak.feature_store.feature_sets.transformations.transformations import (
    BaseTransformation,
    KoalasTransformation,
    PySparkTransformation,
    SparkSqlTransformation,
    UdfTransformation,
)

__all__ = [
    "BaseTransformation",
    "UdfTransformation",
    "KoalasTransformation",
    "PySparkTransformation",
    "SparkSqlTransformation",
    "Window",
    "QwakAggregation",
    "qwak_pandas_udf",
    "Column",
    "Schema",
    "Type",
]
